package ch.ivyteam.q.and.a;

import java.io.IOException;
import java.io.Writer;

import javax.servlet.http.HttpServletRequest;

import ch.ivyteam.ivy.environment.Ivy;
import ch.ivyteam.ivy.request.IHttpRequest;
import ch.ivyteam.ivy.request.IHttpResponse;
import ch.ivyteam.ivy.request.IRequest;
import ch.ivyteam.ivy.request.IResponse;

/**
 * Rest request simplifies access to HTTP request/response.
 * @author Reguel Wermelinger
 */
public class RestRequest {

	public static String getHttpMethod() {
		return getHttpServletRequest().getMethod();
	}

	public static HttpServletRequest getHttpServletRequest() {
		IRequest rootRequest = Ivy.request().getRootRequest();
		if (!(rootRequest instanceof IHttpRequest)) {
			throw new RuntimeException(
					"request must be an HTTP request, but was " + rootRequest);
		}

		IHttpRequest httpRequest = (IHttpRequest) rootRequest;
		HttpServletRequest httpServletRequest = httpRequest.getHttpServletRequest();
		return httpServletRequest;
	}
	
	/**
	 * Ends the request and writes given response content manually. Should be called from a Script activity with no outgoing flow.
	 * @param json
	 */
	public static void sendAsResponse(String json){
		try {
			getHttpResponseWriter().append(json);
		} catch (IOException e) {
			Ivy.log().error("Failed to send json response.", e);
		}
	}

	public static Writer getHttpResponseWriter() {
		IResponse response = Ivy.response();
		if (!(response instanceof IHttpResponse)) {
			throw new RuntimeException(
					"Response must be an HTTP response, but was " + response);
		}

		IHttpResponse httpResponse = (IHttpResponse) response;
		try {
			return httpResponse.getHttpServletResponse().getWriter();
		} catch (IOException ex) {
			throw new RuntimeException("Could not get HTTP response writer.", ex);
		}
	}

}
