package ch.ivyteam.q.and.a;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.List;

import ch.ivyteam.ivy.environment.Ivy;
import ch.ivyteam.ivy.workflow.ITask;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import com.google.gson.reflect.TypeToken;

public class RestService {

	/**
	 * Simple GET request handling: serialize DB pojo as REST -> send it as response
	 * @throws IOException
	 */
	public static void writeToResponse() throws IOException {
		Ivy.log().info(Ivy.session().getSessionUserName());
		List<ch.ivyteam.q.and.a.Person> customers = CrmDao.getCustomers();
		String jsonPerson = new Gson().toJson(customers);
		RestRequest.sendAsResponse(jsonPerson);
	}
	
	/**
	 * Simple POST request handling: deserialize -> store in DB
	 * @param jsonPerson
	 */
	public static void put(String jsonPerson)
	{
		String httpMethod = RestRequest.getHttpMethod();
		if (!httpMethod.equalsIgnoreCase("POST"))
		{
			throw new IllegalStateException("Request method must be POST. But was "+httpMethod);
		}
		
		Person person = new Gson().fromJson(jsonPerson, Person.class);
		Ivy.log().info("got new customer from (REST) PUT request: "+ person);
		CrmDao.addCustomer(person);
	}
	
	/**
	 * Demonstrates how complex core API can be sent as REST response.
	 * @throws Exception
	 */
	public static void writeCoreApi() throws Exception{
		ITask task = Ivy.wfTask();
		String json = new GsonBuilder()
			.registerTypeAdapter(ITask.class, new TaskTypeAdapter())
			.create()
			.toJson(task, new TypeToken<ITask>(){}.getType());
		RestRequest.sendAsResponse(json);
	}
	
	/**
	 * Allows serialization of complex interface type as simple POJO.
	 */
	public static class TaskTypeAdapter implements JsonSerializer<ITask>
	{
		@Override
		public JsonElement serialize(ITask src, Type typeOfSrc,
				JsonSerializationContext context) {
			com.google.gson.JsonObject simpleTask = new com.google.gson.JsonObject();
			simpleTask.add("id", new JsonPrimitive(src.getId()));
			simpleTask.add("state", new JsonPrimitive(src.getState().name()));
			return simpleTask;
		}
	}

}
