package ch.ivyteam.q.and.a;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.junit.Before;
import org.junit.Test;

import simpleRestProvider.Filter;

import com.google.gson.Gson;

public class TestRestMethods {

	private static final int HTTP_PORT = 8081;
	
	@Before
	public void setUp() throws Exception
	{
		storeTestDataInDb();
	}
	
	@Test
	public void restRoundTrip_consumeAndProvideResultAsJson() throws Exception
	{
		CloseableHttpClient httpClient = HttpClientBuilder.create().build();
		HttpPost put = new HttpPost("http://localhost:"+HTTP_PORT+"/ivy/pro/designer/simpleRestProvider/14F82E0248C6C54C/filter.ivp");
		
		Filter filter = new Filter();
		filter.setNameStartsWith("St");
		String json = new Gson().toJson(filter);
		List <NameValuePair> form = new ArrayList <NameValuePair>();
		form.add(new BasicNameValuePair("query", json));
		put.setEntity(new UrlEncodedFormEntity(form));
		
		CloseableHttpResponse response = httpClient.execute(put);
		assertEquals("Request to "+put.getURI()+" failed!", 200, response.getStatusLine().getStatusCode());
		
		String responeContent = IOUtils.toString(response.getEntity().getContent());
		System.out.println("response is: "+responeContent);
		assertFalse(responeContent.isEmpty());
	}
	
	private void storeTestDataInDb() throws Exception
	{
		List<Person> testPersons = generateTestData();
		for(Person testPerson : testPersons)
		{
			CloseableHttpClient httpClient = HttpClientBuilder.create().build();
			HttpPost put = new HttpPost("http://localhost:"+HTTP_PORT+"/ivy/pro/designer/simpleRestProvider/14F82E0248C6C54C/put.ivp");
			
			String json = new Gson().toJson(testPerson);
			List <NameValuePair> form = new ArrayList <NameValuePair>();
			form.add(new BasicNameValuePair("jsonPerson", json));
			put.setEntity(new UrlEncodedFormEntity(form));
			
			CloseableHttpResponse response = httpClient.execute(put);
			assertEquals("Request to "+put.getURI()+" failed.", 
					200, response.getStatusLine().getStatusCode());
		}
	}

	private List<Person> generateTestData() {
		Person p = new Person();
		p.setFirstname("Steve");
		p.setLastname("Washington");
		
		Person p2 = new Person();
		p2.setFirstname("Harry");
		p2.setLastname("Reves");
		
		return Arrays.asList(p, p2);
	}
	
}
